/*******************************************************************************
*
*                        USING TIMER 0 IN PHASECORRECT PWM MODE
*                        --------------------------------------
*
*   Filename:                                    
*   Autor:                  David Kress
*
*   Version:                1.0
*   Date:                   10.11.2014
*   Built with:             ATMEL AVR Studio 4.14 and AVR-GCC; Optimization -01
*
*   Description:            This programm is a simple example how to operate
*                           Timer 0 in phase correct PWM mode. The duty cycle is
*                           set to 64. Every time the timer overflows or
*                           a compare match happens, an interrupt is
*                           executed. The interrupt subroutines toggle  PA0
*                           or PA1 twice. You need an oscilloscope to
*                           observe this signals.
*                           
*
*   What you can learn:     - Start Timer 0 in fast pwm mode
*                           - Set a duty cycle
*                           - Enable the overflow interrupt
*                           - Enable the compare match interrupt
*
*   Clock source:           - Any clock
*
*   Recommended             - High: 0x99
*   Fuse settings:          - Low:  0xE1
*
*
*
*   Schematic:       ___________________
*                   |      ATMEGA32     |
*          Vcc ---->|Vcc                |
*          Vcc ---->|RESET              |
*                   |                   |
*          GND ---->|GND                |
*                   |                   |
*                   |                PB3|----> Phase Correct PWM out
*                   |                PA0|----> DOUT
*                   |                PA1|----> DOUT
*                   |                   |
*                   |                   |
*
*******************************************************************************/


#include <avr/io.h>
#include <avr/interrupt.h>

int main(void)
{

    DDRB = 0x08;                         // Setup PB3 as output
	DDRA = 0x03;                         // Setup PA0 and PA1 as output
    
    TCCR0 |= (1<<WGM00)|(1<<COM01)       // Start timer0 without
             |(1<<CS01);                 // prescaler in non 
                                         // inverting phase correct PWM 
                                         // mode. The clockprescaler is set to 8
    TIMSK |= (1<<OCIE0)|(1<<TOIE0);      // Enable Overflow and Compare Interrupt 


    sei();                               // Set I flag in SREG

    OCR0=63;                             // Set Dutycycle to 25%
  
    for(;;);                             // Endless loop;
                                         // main() will never be left

    return 0;                            // This line will never be executed

}

// Interrupt subroutine timer 0 overflow
ISR(TIMER0_OVF_vect)							
{
    PORTA ^= 0x01;                       // Toggle PA0
	PORTA ^= 0x01;                       // Toggle PA0
}

// Interrupt subroutine timer 0 compare match
ISR(TIMER0_COMP_vect)							
{
    PORTA ^= 0x02;                       // Toggle PA1
	PORTA ^= 0x02;                       // Toggle PA1
}
